@@ -33,7 +33,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QColor>
 #include <QRect>
 #include <QEvent>
+#include <QFutureWatcher>
 #include <QKeyEvent>
+#include <QtConcurrentRun>
 #include <QVector2D>
 
 #include <math.h>
@@ -58,7 +60,7 @@ CubeEffect::CubeEffect()
     , cubeOpacity(1.0)
     , opacityDesktopOnly(true)
     , displayDesktopName(false)
-    , desktopNameFrame(effects->effectFrame(EffectFrameStyled))
+    , desktopNameFrame(NULL)
     , reflection(true)
     , rotating(false)
     , desktopChangedWhileRotating(false)
@@ -95,7 +97,6 @@ CubeEffect::CubeEffect()
 {
     desktopNameFont.setBold(true);
     desktopNameFont.setPointSize(14);
-    desktopNameFrame->setFont(desktopNameFont);
 
     const QString fragmentshader = KGlobal::dirs()->findResource("data", "kwin/cube-reflection.glsl");
     m_reflectionShader = ShaderManager::instance()->loadFragmentShader(ShaderManager::GenericShader, fragmentshader);
@@ -177,31 +178,11 @@ void CubeEffect::loadConfig(QString config)
     invertMouse = conf.readEntry("InvertMouse", false);
     capDeformationFactor = conf.readEntry("CapDeformation", 0) / 100.0f;
     useZOrdering = conf.readEntry("ZOrdering", false);
-    QString file = conf.readEntry("Wallpaper", QString(""));
-    if (wallpaper)
-        wallpaper->discard();
     delete wallpaper;
     wallpaper = NULL;
-    if (!file.isEmpty()) {
-        QImage img = QImage(file);
-        if (!img.isNull()) {
-            wallpaper = new GLTexture(img);
-        }
-    }
     delete capTexture;
     capTexture = NULL;
     texturedCaps = conf.readEntry("TexturedCaps", true);
-    if (texturedCaps) {
-        QString capPath = conf.readEntry("CapPath", KGlobal::dirs()->findResource("appdata", "cubecap.png"));
-        QImage img = QImage(capPath);
-        if (!img.isNull()) {
-            capTexture = new GLTexture(img);
-            capTexture->setFilter(GL_LINEAR);
-#ifndef KWIN_HAVE_OPENGLES
-            capTexture->setWrapMode(GL_CLAMP_TO_BORDER);
-#endif
-        }
-    }
 
     timeLine.setCurveShape(QTimeLine::EaseInOutCurve);
     timeLine.setDuration(rotationDuration);
@@ -262,6 +243,56 @@ CubeEffect::~CubeEffect()
     delete m_cubeCapBuffer;
 }
 
+QImage CubeEffect::loadCubeCap(const QString &capPath)
+{
+    if (!texturedCaps) {
+        return QImage();
+    }
+    return QImage(capPath);
+}
+
+void CubeEffect::slotCubeCapLoaded()
+{
+    QFutureWatcher<QImage> *watcher = dynamic_cast<QFutureWatcher<QImage>*>(sender());
+    if (!watcher) {
+        // not invoked from future watcher
+        return;
+    }
+    QImage img = watcher->result();
+    if (!img.isNull()) {
+        capTexture = new GLTexture(img);
+        capTexture->setFilter(GL_LINEAR);
+#ifndef KWIN_HAVE_OPENGLES
+        capTexture->setWrapMode(GL_CLAMP_TO_BORDER);
+#endif
+        // need to recreate the VBO for the cube cap
+        delete m_cubeCapBuffer;
+        m_cubeCapBuffer = NULL;
+        effects->addRepaintFull();
+    }
+    watcher->deleteLater();
+}
+
+QImage CubeEffect::loadWallPaper(const QString &file)
+{
+    return QImage(file);
+}
+
+void CubeEffect::slotWallPaperLoaded()
+{
+    QFutureWatcher<QImage> *watcher = dynamic_cast<QFutureWatcher<QImage>*>(sender());
+    if (!watcher) {
+        // not invoked from future watcher
+        return;
+    }
+    QImage img = watcher->result();
+    if (!img.isNull()) {
+        wallpaper = new GLTexture(img);
+        effects->addRepaintFull();
+    }
+    watcher->deleteLater();
+}
+
 bool CubeEffect::loadShader()
 {
     if (!(GLPlatform::instance()->supports(GLSL) &&
@@ -294,15 +325,15 @@ bool CubeEffect::loadShader()
         float xmin =  ymin * aspect;
         float xmax = ymax * aspect;
         projection.frustum(xmin, xmax, ymin, ymax, zNear, zFar);
-        cylinderShader->setUniform("projection", projection);
+        cylinderShader->setUniform(GLShader::ProjectionMatrix, projection);
         QMatrix4x4 modelview;
         float scaleFactor = 1.1 * tan(fovy * M_PI / 360.0f) / ymax;
         modelview.translate(xmin * scaleFactor, ymax * scaleFactor, -1.1);
         modelview.scale((xmax - xmin)*scaleFactor / displayWidth(), -(ymax - ymin)*scaleFactor / displayHeight(), 0.001);
-        cylinderShader->setUniform("modelview", modelview);
+        cylinderShader->setUniform(GLShader::ModelViewMatrix, modelview);
         const QMatrix4x4 identity;
-        cylinderShader->setUniform("screenTransformation", identity);
-        cylinderShader->setUniform("windowTransformation", identity);
+        cylinderShader->setUniform(GLShader::ScreenTransformation, identity);
+        cylinderShader->setUniform(GLShader::WindowTransformation, identity);
         QRect rect = effects->clientArea(FullArea, activeScreen, effects->currentDesktop());
         cylinderShader->setUniform("width", (float)rect.width() * 0.5f);
         shaderManager->popShader();
@@ -325,15 +356,15 @@ bool CubeEffect::loadShader()
         float xmin =  ymin * aspect;
         float xmax = ymax * aspect;
         projection.frustum(xmin, xmax, ymin, ymax, zNear, zFar);
-        sphereShader->setUniform("projection", projection);
+        sphereShader->setUniform(GLShader::ProjectionMatrix, projection);
         QMatrix4x4 modelview;
         float scaleFactor = 1.1 * tan(fovy * M_PI / 360.0f) / ymax;
         modelview.translate(xmin * scaleFactor, ymax * scaleFactor, -1.1);
         modelview.scale((xmax - xmin)*scaleFactor / displayWidth(), -(ymax - ymin)*scaleFactor / displayHeight(), 0.001);
-        sphereShader->setUniform("modelview", modelview);
+        sphereShader->setUniform(GLShader::ModelViewMatrix, modelview);
         const QMatrix4x4 identity;
-        sphereShader->setUniform("screenTransformation", identity);
-        sphereShader->setUniform("windowTransformation", identity);
+        sphereShader->setUniform(GLShader::ScreenTransformation, identity);
+        sphereShader->setUniform(GLShader::WindowTransformation, identity);
         QRect rect = effects->clientArea(FullArea, activeScreen, effects->currentDesktop());
         sphereShader->setUniform("width", (float)rect.width() * 0.5f);
         sphereShader->setUniform("height", (float)rect.height() * 0.5f);
@@ -386,9 +417,6 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             }
         }
 
-#ifndef KWIN_HAVE_OPENGLES
-        glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
-#endif
         glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
@@ -552,9 +580,6 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
         glDisable(GL_CULL_FACE);
 
         glDisable(GL_BLEND);
-#ifndef KWIN_HAVE_OPENGLES
-        glPopAttrib();
-#endif
 
         // desktop name box - inspired from coverswitch
         if (displayDesktopName) {
@@ -566,6 +591,10 @@ void CubeEffect::paintScreen(int mask, QRegion region, ScreenPaintData& data)
             QRect screenRect = effects->clientArea(ScreenArea, activeScreen, frontDesktop);
             QRect frameRect = QRect(screenRect.width() * 0.33f + screenRect.x(), screenRect.height() * 0.95f + screenRect.y(),
                                     screenRect.width() * 0.34f, QFontMetrics(desktopNameFont).height());
+            if (!desktopNameFrame) {
+                desktopNameFrame = effects->effectFrame(EffectFrameStyled);
+                desktopNameFrame->setFont(desktopNameFont);
+            }
             desktopNameFrame->setGeometry(frameRect);
             desktopNameFrame->setText(effects->desktopName(frontDesktop));
             desktopNameFrame->render(region, opacity);
@@ -774,11 +803,11 @@ void CubeEffect::paintCap(bool frontFirst, float zOffset)
         m_capShader->setUniform("u_opacity", opacity);
         m_capShader->setUniform("u_mirror", 1);
         if (reflectionPainting) {
-            m_capShader->setUniform("screenTransformation", m_reflectionMatrix * m_rotationMatrix);
+            m_capShader->setUniform(GLShader::ScreenTransformation, m_reflectionMatrix * m_rotationMatrix);
         } else {
-            m_capShader->setUniform("screenTransformation", m_rotationMatrix);
+            m_capShader->setUniform(GLShader::ScreenTransformation, m_rotationMatrix);
         }
-        m_capShader->setUniform("windowTransformation", capMatrix);
+        m_capShader->setUniform(GLShader::WindowTransformation, capMatrix);
         m_capShader->setUniform("u_untextured", texturedCaps ? 0 : 1);
         if (texturedCaps && effects->numberOfDesktops() > 3 && capTexture) {
             capTexture->bind();
@@ -1483,9 +1512,9 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
                 currentShader = sphereShader;
             }
             if (reflectionPainting) {
-                currentShader->setUniform("screenTransformation", m_reflectionMatrix * m_rotationMatrix * origMatrix);
+                currentShader->setUniform(GLShader::ScreenTransformation, m_reflectionMatrix * m_rotationMatrix * origMatrix);
             } else {
-                currentShader->setUniform("screenTransformation", m_rotationMatrix*origMatrix);
+                currentShader->setUniform(GLShader::ScreenTransformation, m_rotationMatrix*origMatrix);
             }
         }
     }
@@ -1495,7 +1524,7 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
             if (mode == Cylinder || mode == Sphere) {
                 shaderManager->popShader();
             } else {
-                shader->setUniform("screenTransformation", origMatrix);
+                shader->setUniform(GLShader::ScreenTransformation, origMatrix);
             }
             shaderManager->popShader();
         }
@@ -1510,9 +1539,6 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
             paint = paint.subtracted(QRegion(w->geometry()));
             // in case of free area in multiscreen setup fill it with cap color
             if (!paint.isEmpty()) {
-#ifndef KWIN_HAVE_OPENGLES
-                glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
-#endif
                 glEnable(GL_BLEND);
                 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                 QVector<float> verts;
@@ -1548,11 +1574,11 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
                     m_capShader->setUniform("u_mirror", 0);
                     m_capShader->setUniform("u_untextured", 1);
                     if (reflectionPainting) {
-                        m_capShader->setUniform("screenTransformation", m_reflectionMatrix * m_rotationMatrix * origMatrix);
+                        m_capShader->setUniform(GLShader::ScreenTransformation, m_reflectionMatrix * m_rotationMatrix * origMatrix);
                     } else {
-                        m_capShader->setUniform("screenTransformation", m_rotationMatrix * origMatrix);
+                        m_capShader->setUniform(GLShader::ScreenTransformation, m_rotationMatrix * origMatrix);
                     }
-                    m_capShader->setUniform("windowTransformation", QMatrix4x4());
+                    m_capShader->setUniform(GLShader::WindowTransformation, QMatrix4x4());
                 }
                 GLVertexBuffer *vbo = GLVertexBuffer::streamingBuffer();
                 vbo->reset();
@@ -1568,9 +1594,6 @@ void CubeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, WindowPa
                     ShaderManager::instance()->popShader();
                 }
                 glDisable(GL_BLEND);
-#ifndef KWIN_HAVE_OPENGLES
-                glPopAttrib();
-#endif
             }
         }
         if (!shader) {
@@ -1884,6 +1907,19 @@ void CubeEffect::setActive(bool active)
         inside->setActive(true);
     }
     if (active) {
+        KConfigGroup conf = effects->effectConfig("Cube");
+        QString capPath = conf.readEntry("CapPath", KGlobal::dirs()->findResource("appdata", "cubecap.png"));
+        if (texturedCaps && !capTexture && !capPath.isEmpty()) {
+            QFutureWatcher<QImage> *watcher = new QFutureWatcher<QImage>(this);
+            connect(watcher, SIGNAL(finished()), SLOT(slotCubeCapLoaded()));
+            watcher->setFuture(QtConcurrent::run(this, &CubeEffect::loadCubeCap, capPath));
+        }
+        QString wallpaperPath = conf.readEntry("Wallpaper", QString(""));
+        if (!wallpaper && !wallpaperPath.isEmpty()) {
+            QFutureWatcher<QImage> *watcher = new QFutureWatcher<QImage>(this);
+            connect(watcher, SIGNAL(finished()), SLOT(slotWallPaperLoaded()));
+            watcher->setFuture(QtConcurrent::run(this, &CubeEffect::loadWallPaper, wallpaperPath));
+        }
         if (!mousePolling) {
             effects->startMousePolling();
             mousePolling = true;
