@@ -362,8 +362,49 @@ void KWinDecorationModule::slotConfigureDecoration()
 
 bool KWinDecorationModule::eventFilter(QObject *o, QEvent *e)
 {
-    if (o == m_ui->decorationList && e->type() == QEvent::Resize)
-        updateScrollbarRange();
+    if (o == m_ui->decorationList) {
+        if (e->type() == QEvent::Resize)
+            updateScrollbarRange();
+        else if (e->type() == QEvent::KeyPress) {
+            int d = 0;
+            const int currentRow = m_ui->decorationList->rootObject()->property("currentIndex").toInt();
+            const int key = static_cast<QKeyEvent*>(e)->key();
+            switch (key) {
+            case Qt::Key_Home:
+                d = -currentRow;
+                break;
+            case Qt::Key_End:
+                d = m_proxyModel->rowCount() - (1 + currentRow);
+                break;
+            case Qt::Key_Up:
+                d = -1;
+                break;
+            case Qt::Key_Down:
+                d = 1;
+                break;
+            case Qt::Key_PageUp:
+            case Qt::Key_PageDown:
+                d = 150;
+                if (QObject *decoItem = m_ui->decorationList->rootObject()->findChild<QObject*>("decorationItem")) {
+                    QVariant v = decoItem->property("height");
+                    if (v.isValid())
+                        d = v.toInt();
+                }
+                if (d > 0)
+                    d = qMax(m_ui->decorationList->height() / d, 1);
+                if (key == Qt::Key_PageUp)
+                    d = -d;
+                break;
+            default:
+                break;
+            }
+            if (d) {
+                d = qMin(qMax(0, currentRow + d), m_proxyModel->rowCount());
+                m_ui->decorationList->rootObject()->setProperty("currentIndex", d);
+                return true;
+            }
+        }
+    }
     return KCModule::eventFilter(o, e);
 }
 
